package com.paraview.parafacebaidu;

import android.accounts.NetworkErrorException;
import android.annotation.TargetApi;
import android.os.Build;
import android.os.Handler;

import org.json.JSONException;
import org.json.JSONObject;

import java.io.BufferedReader;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.OutputStream;
import java.net.HttpURLConnection;
import java.net.URL;
import java.security.cert.CertificateException;
import java.security.cert.X509Certificate;
import java.util.Iterator;
import java.util.Map;

import javax.net.ssl.HostnameVerifier;
import javax.net.ssl.HttpsURLConnection;
import javax.net.ssl.SSLContext;
import javax.net.ssl.SSLSession;
import javax.net.ssl.TrustManager;
import javax.net.ssl.X509TrustManager;

//import cn.paraview.para.common.utils.L;

/**
 * 网络请求框架
 *
 * @author chenqh
 * @email 403167386@qq.com
 * @created 2018/12/28 14:06
 */
public class NetworkUtils {
    /**
     * 请求结果回调接口
     */
    public interface HttpResponseCallBack {

        void onSuccess(JSONObject response) throws JSONException;    //返回数据

        void onFailure();   //网络连接错误 json解析错误
    }

    /**
     * 异步传输post请求 仅文本参数
     *
     * @param url        请求地址
     * @param jsonString 请求参数
     * @param callback   请求回调
     */
    public static void doPost(final String url, final String jsonString, final Map<String, String> paramsHander, final HttpResponseCallBack callback) {
        final Handler handler = new Handler();
        new Thread(new Runnable() {
            @Override
            public void run() {
                final String response = post(url, jsonString, paramsHander);
                handler.post(new Runnable() {
                    @Override
                    public void run() {
                        if (response == null) {
                            callback.onFailure();
                            return;
                        }
                        try {
                            JSONObject json = new JSONObject(response);
                            callback.onSuccess(json);
                        } catch (JSONException e) {
                            callback.onFailure();
                        }
                    }
                });
            }
        }).start();
    }

    /**
     * 异步传输get请求
     *
     * @param url      请求url
     * @param callback 请求回调
     */
    public static void doGet(final String url, final HttpResponseCallBack callback) {
        final Handler handler = new Handler();
        new Thread(new Runnable() {
            @Override
            public void run() {
                final String response = get(url);
                handler.post(new Runnable() {
                    @Override
                    public void run() {
                        if (response == null) {
                            callback.onFailure();
                            return;
                        }
                        try {
                            JSONObject json = new JSONObject(response);
                            callback.onSuccess(json);
                        } catch (JSONException e) {
                            callback.onFailure();
                        }
                    }
                });
            }
        }).start();
    }

    /**
     * 使用HttpURLConnection进行post请求
     *
     * @param url        请求地址
     * @param jsonString 请求参数
     * @return 请求返回值
     */
    @TargetApi(Build.VERSION_CODES.ECLAIR)
    private static String post(String url, String jsonString, Map<String, String> paramsHander) {

        HttpURLConnection conn = null;
        try {
            // 创建url资源
            URL mURL = new URL(url);
            // trust all hosts
            trustAllHosts();
            // 建立http连接
            if (mURL.getProtocol().toLowerCase().equals("https")) {
                HttpsURLConnection https = (HttpsURLConnection) mURL.openConnection();
                https.setHostnameVerifier(DO_NOT_VERIFY);
                conn = https;
            } else {
                conn = (HttpURLConnection) mURL.openConnection();   // 调用URL的openConnection()方法,获取HttpURLConnection对象
            }
            // 设置允许输出
            conn.setDoOutput(true);
            conn.setDoInput(true);
            // 设置不用缓存
            conn.setUseCaches(false);
            // 设置传递方式
            conn.setRequestMethod("POST");
            // 设置维持长连接
            conn.setRequestProperty("Connection", "Keep-Alive");
            // 设置文件字符集:
            conn.setRequestProperty("Charset", "UTF-8");
            // 设置文件长度
            conn.setRequestProperty("Content-Length", String.valueOf(jsonString.length()));
            // 设置文件类型:
            conn.setRequestProperty("Content-Type", "application/json");
            conn.setConnectTimeout(20000);
            conn.setReadTimeout(20000);

            if (paramsHander != null && !paramsHander.isEmpty()) {
                Iterator<Map.Entry<String, String>> iterator = paramsHander.entrySet().iterator();
                while (iterator.hasNext()) {
                    Map.Entry<String, String> param = iterator.next();
                    conn.setRequestProperty(param.getKey(), param.getValue());
                }
            }

            // 开始连接请求
            conn.connect();
            OutputStream out = conn.getOutputStream();
            // 写入请求的字符串
            out.write(jsonString.getBytes());
            out.flush();
            out.close();

            //请求返回
            int responseCode = conn.getResponseCode();// 调用此方法就不必再使用conn.connect()方法
            if (responseCode == HttpURLConnection.HTTP_OK) {
                InputStream is = conn.getInputStream();     //获得输入流
                BufferedReader reader = new BufferedReader(new InputStreamReader(is)); //包装字节流为字符流
                StringBuilder response = new StringBuilder();
                String line;
                while ((line = reader.readLine()) != null) {
                    response.append(line);
                }
                if (is != null) {
                    is.close();
                }

                return response.toString();
            } else {
                throw new NetworkErrorException("response status is " + responseCode);
            }

        } catch (Exception e) {
            e.printStackTrace();
            return null;
        } finally {
            if (conn != null) {
                conn.disconnect();  // 关闭连接
            }
        }
    }


    /**
     * 使用HttpURLConnection进行get请求
     *
     * @param url 请求地址
     * @return 请求返回值
     */
    @TargetApi(Build.VERSION_CODES.ECLAIR)
    private static String get(String url) {
        HttpURLConnection conn = null;
        try {
            //创建连接
            URL mURL = new URL(url);
            // trust all hosts
            trustAllHosts();
            if (mURL.getProtocol().toLowerCase().equals("https")) {
                HttpsURLConnection https = (HttpsURLConnection) mURL.openConnection();
                https.setHostnameVerifier(DO_NOT_VERIFY);
                conn = https;
            } else {
                conn = (HttpURLConnection) mURL.openConnection();   // 调用URL的openConnection()方法,获取HttpURLConnection对象
            }
            conn.setRequestMethod("GET");
            conn.setReadTimeout(5000);
            conn.setConnectTimeout(10000);

            //请求返回
            int responseCode = conn.getResponseCode();
            if (responseCode == HttpURLConnection.HTTP_OK) {
                InputStream is = conn.getInputStream();     //获得输入流
                BufferedReader reader = new BufferedReader(new InputStreamReader(is));      //包装字节流为字符流
                StringBuilder response = new StringBuilder();
                String line;
                while ((line = reader.readLine()) != null) {
                    response.append(line);
                }
                if (is != null) {
                    is.close();
                }

                return response.toString();
            } else {
                throw new NetworkErrorException("response status is " + responseCode);
            }

        } catch (Exception e) {
            e.printStackTrace();
            return null;
        } finally {
            if (conn != null) {
                conn.disconnect();  // 关闭连接
            }
        }
    }

    public final static HostnameVerifier DO_NOT_VERIFY = new HostnameVerifier() {

        public boolean verify(String hostname, SSLSession session) {
            return true;
        }
    };

    /**
     * Trust every server - dont check for any certificate
     */
    public static void trustAllHosts() {
        final String TAG = "trustAllHosts";
        // Create a trust manager that does not validate certificate chains
        TrustManager[] trustAllCerts = new TrustManager[]{new X509TrustManager() {

            public X509Certificate[] getAcceptedIssuers() {
                return new X509Certificate[]{};
            }

            public void checkClientTrusted(X509Certificate[] chain, String authType) throws CertificateException {
                L.i("checkClientTrusted");
            }

            public void checkServerTrusted(X509Certificate[] chain, String authType) throws CertificateException {
                L.i("checkServerTrusted");
            }
        }};

        // Install the all-trusting trust manager
        try {
            SSLContext sc = SSLContext.getInstance("TLS");
            sc.init(null, trustAllCerts, new java.security.SecureRandom());
            HttpsURLConnection.setDefaultSSLSocketFactory(sc.getSocketFactory());
        } catch (Exception e) {
            e.printStackTrace();
        }
    }
}
