//
//  NSBundle+ParaFace.m
//  Face_SDK
//
//  Created by ZhangSan on 2019/5/20.
//  Copyright © 2019 PaiLa. All rights reserved.
//

#import "NSBundle+ParaFace.h"
#define kParaFaceBundleName @"LocalLanguage.bundle"
#define kParaFaceLanguageStyle @"DSADLanguageStyle"

@implementation NSBundle (ParaFace)

+ (instancetype)KOD_localizableBundleWithBundleName:(NSString *)bundleName{
    static NSBundle *localizableBundle = nil;
    if (localizableBundle == nil) {
        if (!bundleName) {
            bundleName = kParaFaceBundleName;
        }
        NSString *bundleType = nil;
        if (bundleName && ![bundleName hasSuffix:@"bundle"]) {
            bundleType = @"bundle";
        }
        NSString *bundlePath = [[NSBundle mainBundle] pathForResource:bundleName ofType:bundleType];
        localizableBundle = [NSBundle bundleWithPath:bundlePath];
    }
    return localizableBundle;
}

+ (NSString *)KOD_localizedStringForKey:(NSString *)key{
    return [self KOD_localizedStringForKey:key value:nil];
}

+ (NSString *)KOD_localizedStringForKey:(NSString *)key value:(NSString *)value{
    NSBundle *bundle = nil;
    //NSString *language = [self getLanguageFromSystem];
    //NSString *language = [self getLanguageFromPlist];
    NSString * language = [self getLanguageFromDevelopersSetup];
    if (!language) {
        language = [self getLanguageFromSystem];
    }
    //从FrameworkTestBundle.bundle中查找资源
    NSString *bundlePath = [[NSBundle KOD_localizableBundleWithBundleName:nil] pathForResource:language ofType:@"lproj"];
    bundle = [NSBundle bundleWithPath:bundlePath];
    value = [bundle localizedStringForKey:key value:value table:nil];
    return [[NSBundle mainBundle] localizedStringForKey:key value:value table:nil];
}

//这个设置语言是通过读取当前系统使用语言
+ (NSString *)getLanguageFromSystem{
    NSString *language = [NSLocale preferredLanguages].firstObject;
    if ([language hasPrefix:@"en"]) {
        language = @"en";
    } else if ([language hasPrefix:@"zh"]) {
        language = @"zh-Hans"; // 简体中文
    } else {
        language = @"en";
    }
    return language;
}


//这个是设置语言通过开发者手动调用，从NSUserDefaults里面去读kDSADLanguageStyle这个字段是哪一种语言
+ (NSString *)getLanguageFromDevelopersSetup{
    NSUserDefaults *userDefaults = [NSUserDefaults standardUserDefaults];
    NSInteger languageStyle = [[userDefaults valueForKey:kParaFaceLanguageStyle] integerValue];
    if (!languageStyle) {
        return nil;
    }
    switch (languageStyle) {
        case 1:
            return @"en";
            break;
        case 2:
            return @"zh-Hans";
            break;
        default:
            return @"en";
            break;
    }
}

@end
