//
//  ParaFaceNetAccessModel.m
//  ParaFaceDemo
//
//  Created by ZhangSan on 2018/11/24.
//  Copyright © 2018年 PaiLa. All rights reserved.
//

#import "ParaFaceNetAccessModel.h"
#import "NSString+Additions.h"
#import <CommonCrypto/CommonDigest.h>

#define BASE_URL @"https://aip.baidubce.com"
#define ACCESS_TOEKN_URL [NSString stringWithFormat:@"%@/oauth/2.0/token",BASE_URL]
#define REG_URL [NSString stringWithFormat:@"%@/rest/2.0/face/v3/faceset/user/add",BASE_URL]
#define SEARCH_URL [NSString stringWithFormat:@"%@/rest/2.0/face/v3/search",BASE_URL]

@interface ParaFaceNetAccessModel ()

@property (nonatomic, readwrite, retain) NSString *accessToken;
@property (nonatomic, readwrite, retain) NSString *groupID;
@property (nonatomic, readwrite, retain) NSString *userID;

@end

@implementation ParaFaceNetAccessModel

+ (instancetype)sharedInstance {
    static ParaFaceNetAccessModel *instance = nil;
    static dispatch_once_t onceToken;
    dispatch_once(&onceToken, ^{
        instance = [[ParaFaceNetAccessModel alloc] init];
    });
    return instance;
}

- (instancetype)init {
    if (self = [super init]) {
//        _groupID = [@"123456" md5String];
        _groupID = @"123456";
        _userID = @"1234567";
    }
    return self;
}

- (void)getAccessTokenWithAK:(NSString *)ak SK:(NSString *)sk {
    [[ParaFaceManager sharedInstance] PostWithURL:ACCESS_TOEKN_URL Params:@{@"grant_type":@"client_credentials",@"client_id":ak,@"client_secret":sk} completion:^(NSError * _Nonnull error, id  _Nonnull resultObject) {
        if (error == nil) {
            NSDictionary* dict = [NSJSONSerialization JSONObjectWithData:resultObject options:NSJSONReadingAllowFragments error:nil];
            NSLog(@"人脸第一次dict%@",dict);
            NSString *faceAccessTokenString = [NSString stringWithFormat:@"%@",dict[@"access_token"]];
            [[NSUserDefaults standardUserDefaults] setValue:faceAccessTokenString forKey:@"faceAccessTokenString"];
            [[NSUserDefaults standardUserDefaults] synchronize];
        } else {
            NSLog(@"error=%@",error);
        }
    }];
    

}

- (void)setLocalDataUserName:(NSString *)username userID:(NSString *)userid groupID:(NSString *)groupid
{
    [[NSUserDefaults standardUserDefaults] setObject:username forKey:@"FaceSDKParaUserName"];
    [[NSUserDefaults standardUserDefaults] setObject:userid forKey:@"FaceSDKParaUserID"];
    [[NSUserDefaults standardUserDefaults] setObject:groupid forKey:@"FaceSDKParaGroupID"];
    [[NSUserDefaults standardUserDefaults] synchronize];
    
}

- (void)registerFaceWithImageBaseString:(NSString *)imageStr completion:(FinishBlockWithObject)completionBlock
{
    NSString *userIDString = [[NSUserDefaults standardUserDefaults] objectForKey:@"FaceSDKParaUserID"];
    NSString *userNameString = [[NSUserDefaults standardUserDefaults] objectForKey:@"FaceSDKParaUserName"];
    NSString *groupIDString =  [[NSUserDefaults standardUserDefaults] objectForKey:@"FaceSDKParaGroupID"];
    NSString *faceTokenStirng = [[NSUserDefaults standardUserDefaults] objectForKey:@"faceAccessTokenString"];
    NSDictionary* parm = @{@"user_id":userIDString,
                           @"user_info":userNameString,
                           @"group_id":groupIDString ?: @"",
                           @"image_type":@"BASE64",
                           @"liveness_control":@"NORMAL",
                           @"quality_control":@"NORMAL",
                           @"image":imageStr};
    
    [[ParaFaceManager sharedInstance] PostWithURL:[NSString stringWithFormat:@"%@?access_token=%@",REG_URL,faceTokenStirng] Params:parm completion:^(NSError * _Nonnull error, id  _Nonnull resultObject) {
        completionBlock(error,resultObject);
    }];
}


- (void)searchFaceWithImageBaseString:(NSString *)imageStr userID:(NSString *)userID groupID:(NSString *)groupID completion:(FinishBlockWithObject)completionBlock
{
//    NSString *userIDString = [[NSUserDefaults standardUserDefaults] objectForKey:@"FaceSDKParaUserID"];
//    NSString *groupIDString =  [[NSUserDefaults standardUserDefaults] objectForKey:@"FaceSDKParaGroupID"];
    NSString *faceTokenStirng = [[NSUserDefaults standardUserDefaults] objectForKey:@"faceAccessTokenString"];
    NSMutableDictionary* parm = [@{@"image":imageStr,
                                   @"image_type":@"BASE64",
                                   @"liveness_control":@"NORMAL",
                                   @"quality_control":@"NORMAL",
                                   @"group_id_list":groupID ?: @"",
                                   @"user_id":userID ?: @""
                                   } mutableCopy];
    [[ParaFaceManager sharedInstance] PostWithURL:[NSString stringWithFormat:@"%@?access_token=%@",SEARCH_URL,faceTokenStirng] Params:parm completion:^(NSError * _Nonnull error, id  _Nonnull resultObject) {
        completionBlock(error,resultObject);
    }];
}


#pragma mark - MD5加密
- (NSString *)jiaMiStr:(NSString *)str
{
    NSString * varifyStr1 = @"!@#$";
    NSString * varifyStr2 = @"*&^%";
    NSString * mdStr = [NSString stringWithFormat:@"%@%@%@",varifyStr1,str,varifyStr2];
    
    const char *cStr = [mdStr UTF8String];
    unsigned char digest[CC_MD5_DIGEST_LENGTH];
    CC_MD5(cStr, strlen(cStr), digest);
    
    NSMutableString *result = [NSMutableString stringWithCapacity:CC_MD5_DIGEST_LENGTH *2];
    for(int i =0; i < CC_MD5_DIGEST_LENGTH; i++)
        [result appendFormat:@"%02x", digest[i]];
    
    return [result lowercaseString];
}

@end
