import 'dart:async';

import 'package:flutter/services.dart';
import 'faceuser.dart';

class Parafacebaidu {
  static const MethodChannel _channel =
      const MethodChannel('parafacebaidu');

  static Future<String> get platformVersion async {
    final String version = await _channel.invokeMethod('getPlatformVersion');
    return version;
  }
  Future<LivenessResult> liveness({language = 'zh'}) async {
    var arguments = Map();
    arguments['language'] = language;
    final Map<dynamic, dynamic> map =
    await _channel.invokeMethod('liveness', arguments);
    return map != null ? new LivenessResult.fromMap(map) : null;
  }

  Future<DetectResult> detect({language = 'zh'}) async {
    var arguments = Map();
    arguments['language'] = language;
    final Map<dynamic, dynamic> map =
    await _channel.invokeMethod('detect', arguments);
    return map != null ? new DetectResult.fromMap(map) : null;
  }


  static Future<bool> get initLib async {
    Map lisenceMap = {
      'licenseID': 'esc-face-android',
      'licenseFileName': 'idl-license.face-android'
    };
    final bool version = await _channel.invokeMethod('init', lisenceMap);
    return version;
  }

  static Future<Map> faceResgiter(FaceUser user) async {
    final result = await _channel.invokeMapMethod('faceResgiter', user.toMap());
    print(result);
    return result;
  }

  static Future<Map> faceSearch(FaceUser user) async {
    final result = await _channel.invokeMapMethod('faceSearch', user.toMap());
    print(result);
    return result;
  }
}

class LivenessResult {
  LivenessResult({this.success, this.image});

  factory LivenessResult.fromMap(Map<dynamic, dynamic> map) =>
      new LivenessResult(
        success: map['success'],
        image: map['image'],
      );

  final String success;

  // sucess=true
  final String image;

  @override
  String toString() => 'LivenessResult: $success,$image';
}

class DetectResult {
  DetectResult({this.success, this.image});

  factory DetectResult.fromMap(Map<dynamic, dynamic> map) => new DetectResult(
    success: map['success'],
    image: map['image'],
  );

  final String success;

  // sucess=true
  final String image;

  @override
  String toString() => 'DetectResult: $success,$image';
}
